"""Model classes for PyQt's model view architecture.

Copyright (C) 2021  Andreas Hellerschmied <andreas.hellerschmied@bev.gv.at>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

from PyQt5.QtCore import QAbstractTableModel, Qt
from PyQt5.QtWidgets import QMessageBox
import datetime as dt


NONE_REPRESENTATION_IN_TABLE_VIEW = ''  # Representation of None values in table views in the GUI


class SetupTableModel(QAbstractTableModel):
    """Model for displaying the station data.

    Attributes
    ----------
    _surveys : dict of py.obj:`gravtools.survey.Survey`
        Dict of survey object that are assigne by reference to the campaing data object
        (py.obj:`gravtools.survey.Campaign`).
    _data : pandas dataframe
        Data frame that holds the observation data of a specific setup or survey to be displayed in the table view.
    _data_survey_name : str
        Name of the currently selected survey.
    """

    # Columns that will be shown in the table view, if available in the data (Also defines the order of columns):
    # - keys: Actual names of the dataframe columns
    # - items: Header names for the Table View Widget
    _SHOW_COLUMNS_IN_TABLE_DICT = {
        'station_name': 'Station',
        'setup_id': 'Setup ID',
        'epoch_dt': 'Epoch UTC',
        'g_mugal': 'g [µGal]',
        'sd_g_mugal': 'SD [µGal]',
        'epoch_unix': 'Epoch Unix',
        'delta_t_h': 'd_t [h]',
        'delta_t_campaign_h': 'd_t camp [h]',
        'sd_setup_mugal': 'SD of obs [µGal]',
        'number_obs': 'Number of obs.',
        'dhf_sensor_m': 'dhf_sensor [m]',
        'linear_scale': 'linear scale',
    }
    _SHOW_COLUMNS_IN_TABLE = list(_SHOW_COLUMNS_IN_TABLE_DICT.keys())  # Actual list of columns to be shown

    # List of columns that are shown in the simple mode of the GUI
    _SHOW_COLUMNS_IN_TABLE_SIMPLE_GUI = [
        'station_name',
        'setup_id',
        'g_mugal',
        'sd_g_mugal',
        'epoch_dt',
    ]

    _DECIMAL_PLACES_PER_FLOAT_COLUMN = {  # key = column name; value = number of decimal places
        'g_mugal': 1,
        'sd_g_mugal': 1,
        'epoch_unix': 1,
        'delta_t_h': 3,
        'delta_t_campaign_h': 3,
        'sd_setup_mugal': 1,
        'dhf_sensor_m': 3,
        'linear_scale': 5,
    }

    def __init__(self, surveys):
        """Initialize the observation table view model.

        Parameters
        ----------
        surveys : dict of py.obj:`gravtools.survey.Survey` objects
        """
        QAbstractTableModel.__init__(self)
        self._surveys = {}
        self._data = None  # Observations (or at subset of them) of the survey with the name `self._data_survey_name`
        self.load_surveys(surveys)
        self._data_survey_name = ''  # Name of the Survey that is currently represented by `self._data`
        self._data_column_names = None
        self.flag_gui_simple_mode = False
        self._reference_height_type = ''
        self._tide_correction_type = ''
        self._atm_pres_correction_type = ''
        self._scale_correction_type = ''
        self._oceanload_correction_type = ''

    def reset_model(self):
        """Reset the model to a state without selected data to display."""
        self._data = None  # Observations (or at subset of them) of the survey with the name `self._data_survey_name`
        self._data_survey_name = ''  # Name of the Survey that is currently represented by `self._data`
        self._data_column_names = None
        self._reference_height_type = ''
        self._tide_correction_type = ''
        self._atm_pres_correction_type = ''
        self._scale_correction_type = ''
        self._oceanload_correction_type = ''

    def load_surveys(self, surveys):
        """Load observation data (dict of survey objects in the campaign object) to the observation model.

        Notes
        -----
        The data is assigned by reference, i.e. all changes in `_surveys` will propagate to the data origin.
        """
        if surveys is None:
            self._surveys = {}
            return
        self._surveys = surveys

    def update_view_model(self, survey_name, setup_id, gui_simple_mode=False):
        """Update the `_data` DataFrame that hold the actual data that is displayed."""
        if survey_name is not None:
            try:
                setup_df = self._surveys[survey_name].setup_df
            except KeyError:
                QMessageBox.critical(self.parent(), 'Error!', f'Survey "{survey_name}" is not available in this campaign.')
            else:
                self._data_survey_name = survey_name
                try:
                    self._reference_height_type = self._surveys[survey_name].setup_reference_height_type
                    self._tide_correction_type = self._surveys[survey_name].setup_tide_correction_type
                except AttributeError:
                    self._reference_height_type = ''
                    self._tide_correction_type = ''
                try:
                    self._atm_pres_correction_type = self._surveys[survey_name].setup_atm_pres_correction_type
                except AttributeError:
                    self._atm_pres_correction_type = ''
                try:
                    self._scale_correction_type = self._surveys[survey_name].setup_scale_correction_type
                except AttributeError:
                    self._scale_correction_type = ''
                try:
                    self._oceanload_correction_type = self._surveys[survey_name].setup_oceanload_correction_type
                except AttributeError:
                    self._oceanload_correction_type = ''
                self.flag_gui_simple_mode = gui_simple_mode

                # Get list of columns to be depicted via the table model:
                # - Keep order of items in `self._SHOW_COLUMNS_IN_TABLE`
                if setup_df is not None:
                    setup_df_columns_set = frozenset(setup_df.columns)
                    table_model_columns = [x for x in self.get_table_columns if x in setup_df_columns_set]

                if setup_id is None:  # No setup ID provided => Take all observations in survey
                    if setup_df is None:
                        self._data = None
                        self._data_column_names = None
                    else:
                        # self._data = setup_df.copy(deep=True)
                        self._data = setup_df.loc[:, table_model_columns].copy(deep=True)
                        self._data_column_names = self._data.columns.to_list()
                else:  # Only take observations of the specified setup
                    if setup_df is None:
                        self._data = None
                        self._data_column_names = None
                    else:
                        # self._data = setup_df[setup_df['setup_id'] == setup_id].copy(deep=True)
                        self._data = setup_df.loc[setup_df['setup_id'] == setup_id, table_model_columns].copy(deep=True)
                        self._data_column_names = self._data.columns.to_list()
        else:
            self.reset_model()

    def headerData(self, section, orientation, role):
        # section is the index of the column/row.
        if self._data is not None:  # No data available yet
            if role == Qt.DisplayRole:
                if orientation == Qt.Horizontal:
                    return self._SHOW_COLUMNS_IN_TABLE_DICT[str(self._data.columns[section])]
                if orientation == Qt.Vertical:
                    return str(self._data.index[section])

    def rowCount(self, parent=None):
        if self._data is not None:
            return self._data.shape[0]
        else:
            return 0

    def columnCount(self, parent=None):
        if self._data is not None:
            return self._data.shape[1]
        else:
            return 0

    def data(self, index, role=Qt.DisplayRole):
        if self._data is not None:
            if index.isValid():
                if role == Qt.DisplayRole or role == Qt.UserRole:
                    value = self._data.iloc[index.row(), index.column()]
                    # Custom formatter (string is expected as return type):
                    if value is None:  #
                        return NONE_REPRESENTATION_IN_TABLE_VIEW
                    elif isinstance(value, float):
                        if value != value:  # True, if value is "NaN"
                            return NONE_REPRESENTATION_IN_TABLE_VIEW
                        else:
                            try:
                                col_name_str = self._data_column_names[index.column()]
                                num_dec_places = self._DECIMAL_PLACES_PER_FLOAT_COLUMN[col_name_str]
                                return '{1:.{0}f}'.format(num_dec_places, value)
                            except KeyError:
                                return str(value)
                    elif isinstance(value, dt.datetime):
                        return value.strftime("%Y-%m-%d, %H:%M:%S")
                    else:  # all other
                        return str(value)

                if role == Qt.TextAlignmentRole:
                    value = self._data.iloc[index.row(), index.column()]
                    if isinstance(value, int) or isinstance(value, float):
                        # Align right, vertical middle.
                        return Qt.AlignVCenter + Qt.AlignRight

    @property
    def get_table_columns(self):
        """Returns a list with all columns names of the dataframe that should be copied to the view model."""
        if self.flag_gui_simple_mode:
            return [value for value in self._SHOW_COLUMNS_IN_TABLE if value in self._SHOW_COLUMNS_IN_TABLE_SIMPLE_GUI]
        else:
            return self._SHOW_COLUMNS_IN_TABLE

    @property
    def get_ref_heigth_type(self):
        """Returns the reference height type of the setup observations."""
        return self._reference_height_type

    @property
    def get_tidal_corr_type(self):
        """Returns the tidal corrections applied on the setup observations."""
        return self._tide_correction_type

    @property
    def get_atm_pres_corr_type(self):
        """Returns the atmospheric pressure corrections applied on the setup observations."""
        return self._atm_pres_correction_type

    @property
    def get_scale_corr_type(self):
        """Returns the scale corrections applied on the setup observations."""
        return self._scale_correction_type

    @property
    def get_oceanload_corr_type(self):
        """Returns the ocean-loading correction applied on the setup observations."""
        return self._oceanload_correction_type



"""Processing gravity surveys by the BEV legacy method.

Contains classes for the legacy processing scheme of gravimeter observations at BEV using multiple linear regression for
drift adjustment. This scheme also involves the determination of absolute gravity values based on datum stations.

Copyright (C) 2021  Andreas Hellerschmied <andreas.hellerschmied@bev.gv.at>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

import pandas as pd
import numpy as np
# from scipy import stats
import datetime as dt
import pytz
# import copy
from sklearn.linear_model import LinearRegression

from gravtools.models.lsm import LSM
# from gravtools import settings
from gravtools import __version__ as GRAVTOOLS_VERSION


def prep_polyval_coef(pol_coef_dict):
    """Preparation of the polynomial coefficients for np.polyval().

    Parameters
    ----------
    pol_coef_dict : dict
        Dictionary with polynomial coefficients from mlr.

    Returns
    -------
    list : list of polynomial coefficients suitable for np.polyval()
    """
    poly_coef = list()
    for degree, value in pol_coef_dict.items():
        poly_coef.append(value)
    poly_coef.reverse()
    poly_coef.append(0)
    return poly_coef


class BEVLegacyProcessing(LSM):
    """Legacy processing scheme for relative gravimeter observations at BEV.

    First, the instrumental drift is estimated by multiple linear regression. Afterwards, the drift-corrected
    gravimeter readings per station are referred to absolute gravity values at the datum stations.

    Attributes
    ----------

    """

    # Column names of self.drift_pol_df:
    # - keys: Column names of the pandas dataframe
    # - values: Short description for table headers, etc., in the GUI
    _DRIFT_POL_DF_COLUMNS_DICT = {
        'survey_name': 'Survey',
        'degree': 'Degree',
        'coefficient': 'Coefficient',
        'sd_coeff': 'SD',
        'coeff_unit': 'Unit',
        'ref_epoch_t0_dt': 't0',
    }
    _DRIFT_POL_DF_COLUMNS = list(_DRIFT_POL_DF_COLUMNS_DICT.keys())

    def __init__(self, stat_df, setups, comment='', write_log=True):
        """
        Parameters
        ----------
        comment : str, optional (default = '')
            Arbitrary comment on the LSM run.
        write_log : bool, optional (default=True)
            Flag that indicates whether log string should be written or not.
        """
        # Call constructor from abstract base class:
        lsm_method = 'MLR_BEV'
        super().__init__(lsm_method, stat_df=stat_df, setups=setups, comment=comment, write_log=write_log)

    @classmethod
    def from_campaign(cls, campaign, comment='', write_log=True):
        """Constructor that generates and populates the LSM object from a Campaign class object.

        Notes
        -----
        Put checks that are dependent on the LSM method into here! All common checks and preparations are implemented
        in the `from_campaign` method of the parent class `LSM`.

        Parameters
        ----------
        campaign : :py:obj:`gravtools.models.survey.Campaign`
            The campaign object needs to provide setup data for all active surveys and the related station data.
        comment : str, optional (default = '')
            Arbitrary comment on the LSM run.
        write_log : bool, optional (default=True)
            Flag that indicates whether log string should be written or not.

        Returns
        -------
        :py:obj:`.BEVLegacyProcessing`
            Contains all information required for adjusting the campaign.
        """
        return super().from_campaign(campaign, comment='', write_log=True)

    def adjust(self, drift_pol_degree=1,
               verbose=False
               ):  # Confidence level):
        """Run the adjustment.

        Parameters
        ----------
        drift_pol_degree : int, optional (default=1)
            Degree of estimated drift polynomial.
        verbose : bool, optional (default=False)
            If True, status messages are printed to the command line, e.g. for debugging and testing

        Notes
        -----
        Only one drift polynomial is adjusted in the input data! Hence, it might be problematic to use observations
        of multiple surveys that were taken over a longer time period.
        """

        # ##### 1.) Initial preparations #####
        if len(self.setups) > 1:
            raise RuntimeError(f'The current campaign contains {len(self.setups)} active surveys. The "MLR" method'
                               f'only supports the adjustment of single surveys. Use a different adjustment method or '
                               f'adjust all surveys individually.')

        # - Observations and parameters:
        all_setups_df = None
        observed_stations = []
        survey_names = []
        number_of_observations = 0
        setup_ids = []
        ref_epoch_t0_dt_list = []
        for survey_name, setup_data in self.setups.items():
            setup_df = setup_data['setup_df']
            observed_stations = observed_stations + setup_df['station_name'].to_list()
            number_of_observations = number_of_observations + len(setup_df)
            survey_names.append(survey_name)
            setup_ids = setup_ids + setup_df['setup_id'].to_list()
            if all_setups_df is None:
                all_setups_df = setup_df.copy(deep=True)
                all_setups_df['survey_name'] = survey_name
            else:
                tmp_setup_df = setup_df.copy(deep=True)
                tmp_setup_df['survey_name'] = survey_name
                all_setups_df = pd.concat([all_setups_df, tmp_setup_df]).reset_index()  # concat dataframes
            ref_epoch_t0_dt_list.append(setup_data['ref_epoch_delta_t_campaign_h'])
        observed_stations = list(
            set(observed_stations))  # Unique list of stations => order of stations in matrices!
        number_of_stations = len(observed_stations)
        number_of_surveys = len(self.setups)
        # number_of_parameters = number_of_stations + drift_pol_degree * number_of_surveys  # Total number of parameters to be estimated
        # number_of_diff_obs = number_of_observations - number_of_surveys

        # Check if the drift polynomial reference epoch refers to the same time (campaign start) for all surveys:
        if len(set(ref_epoch_t0_dt_list)) > 1:
            raise AssertionError(f'The drift polynomial reference epochs do not match!')
        ref_epoch_t0_dt = ref_epoch_t0_dt_list[0]

        # Check, if setup IDs are unique:
        self.check_unique_setups(setup_ids)

        # Write log:
        if verbose or self.write_log:
            time_now_str = dt.datetime.now(tz=pytz.UTC).strftime('%Y-%m-%d, %H:%M:%S %Z')
            tmp_str = f'#### Adjustment log (MLR, BEV legacy) ####\n'
            tmp_str += f'Processed with GravTools {GRAVTOOLS_VERSION} ({time_now_str})\n'
            tmp_str += f'Comment: {self.comment}\n'
            tmp_str += f'\n'
            tmp_str += f'---- Input data and settings ----\n'
            tmp_str += f'Number of surveys: {number_of_surveys}\n'
            tmp_str += f'Number of stations: {number_of_stations}\n'
            tmp_str += f'Number of observations: {number_of_observations}\n'
            # tmp_str += f'Number of estimated parameters: {number_of_parameters}\n'  # obsolet
            # tmp_str += f'Number of datum stations: {number_of_datum_stations}\n'  # ????????????????????ß
            # tmp_str += f'Degree of freedom: {number_of_diff_obs - number_of_parameters}\n'  # obsolet
            tmp_str += f'\n'
            tmp_str += f'---- Survey infos ----\n'
            tmp_str += self.survey_info_string
            tmp_str += f'\n'
            tmp_str += f'\n'
            if verbose:
                print(tmp_str)
            if self.write_log:
                self.log_str += tmp_str

        # Get dataframe with subset of observed stations only:
        filter_tmp = self.stat_df['station_name'].isin(observed_stations)
        self.stat_obs_df = self.stat_df.loc[filter_tmp].copy(deep=True)  # All observed stations

        # ##### 2.) Drift Adjustment #####
        # All obs data is here: all_setups_df['delta_t_campaign_h']
        # - delta_t [h] since first obs in campaign => all_setups_df['delta_t_campaign_h']
        # Test dataset: "n20200701_1" and "2020-07-01_hermannskogel.txt" (use last obs of each setup! => then equal!)

        # Set up obs_df with categorical variables for each point:
        df_short = all_setups_df[['station_name', 'g_mugal', 'delta_t_campaign_h']].copy(deep=True)

        # Create obs_df with categorical dummy variables for each station name:
        prefix = 'is_pkt'
        df_dummy = pd.get_dummies(df_short, prefix=[prefix])
        categorical_variables = [prefix + '_' + str1 for str1 in observed_stations]

        # Set up target parameters according to polynomial degree:
        target_parameters = []
        for i in range(1, drift_pol_degree + 1):  # i = 1 to polynomial_degree
            col_name = 'dt_{}'.format(i)
            df_dummy[col_name] = df_dummy['delta_t_campaign_h'].pow(i)
            target_parameters.append(col_name)

        selection_list = target_parameters + categorical_variables

        # Calculate regression:
        mlr = LinearRegression(fit_intercept=False)  # No b0 coefficient estimated
        mlr.fit(df_dummy[selection_list], df_dummy['g_mugal'])

        pol_coef_dict = {i + 1: mlr.coef_[i] for i in range(0, drift_pol_degree)}  # Dict with polynomial coefficients

        # Store drift-corrected gravitymeter readings ("g_drift_est_mugal"):
        tmp_list = mlr.coef_[-number_of_stations:]  #
        for idx, station_name in enumerate(observed_stations):
            filter_tmp = self.stat_obs_df['station_name'] == station_name
            self.stat_obs_df.loc[filter_tmp, 'g_drift_est_mugal'] = tmp_list[idx]

        # Calculate drift correction for all observations:
        poly_coef = prep_polyval_coef(pol_coef_dict)

        all_setups_df['corr_drift_mugal'] = np.polyval(poly_coef, all_setups_df['delta_t_campaign_h'])

        # Calculate statistics:
        for station_name in observed_stations:
            # Estimate for this station:
            filter_tmp_stat_obs_df = self.stat_obs_df['station_name'] == station_name
            g_drift_est_mugal = self.stat_obs_df.loc[filter_tmp_stat_obs_df, 'g_drift_est_mugal'].values[0]

            # Calculate difference between drift corrected reading and the estimated gravity reading at this point:
            filter_tmp_all_setups_df = all_setups_df['station_name'] == station_name
            all_setups_df.loc[filter_tmp_all_setups_df, 'abw_mugal'] = \
                all_setups_df.loc[filter_tmp_all_setups_df, 'g_mugal'] - \
                all_setups_df.loc[filter_tmp_all_setups_df, 'corr_drift_mugal'] - g_drift_est_mugal

            # Sigma of "g_drift_est_mugal":
            number_of_observations_of_station = filter_tmp_all_setups_df[filter_tmp_all_setups_df].count()
            if number_of_observations_of_station > 1:  # Get all drift-calibration points
                self.stat_obs_df['is_drift_point'] = True
                # Warum "+4"? Eher willkürlich in DRIFT2011 gesetzt, oder? 4 = 2*2.
                self.stat_obs_df.loc[filter_tmp_stat_obs_df, 'sig_g_drift_est_mugal'] = \
                    np.sqrt((sum(all_setups_df.loc[filter_tmp_all_setups_df, 'abw_mugal'] ** 2) /
                             (number_of_observations_of_station - 1)) + 4)
            else:
                self.stat_obs_df['is_drift_point'] = False

        pol_coef_sig_mugal = np.sqrt(
            sum(all_setups_df['abw_mugal'] ** 2) / (
                    number_of_observations - 1 - sum(self.stat_obs_df['is_drift_point'] == False)))

        # Sigma for all estimats with only one observation:
        # - Why + 25? Just an assumption (measuremenet accuracy is 5 µGal => 5**2 = 25)?
        self.stat_obs_df.loc[~self.stat_obs_df['is_drift_point'],
                             'sig_g_drift_est_mugal'] = np.sqrt(pol_coef_sig_mugal ** 2 + 25)

        # Write log:
        if verbose or self.write_log:
            tmp_str = f'#### Drift adjustment (MLR) ####\n'
            tmp_str += f'\n'
            tmp_str += f'Degree of drift polynomial: {drift_pol_degree}\n'
            tmp_str += f'Drift polynomial reference epoch: {ref_epoch_t0_dt.strftime("%Y-%m-%d, %H:%M:%S")}\n'
            tmp_str += f'\n'
            tmp_str += f'Polynomial coefficients (sigma = {pol_coef_sig_mugal:5.2f} µGal)\n'
            for degree, value in pol_coef_dict.items():
                tmp_str += f' - b{degree:} = {value:9.5f} (µGal/h)^{degree} ({value*24/1000:9.5f} (mGal/Tag)^{degree})\n'
            tmp_str += f'\n'
            tmp_str += f'Drift-corrected gravimeter reading per station:\n'
            for idx, row in self.stat_obs_df.iterrows():
                tmp_str += f' - {row["station_name"]:12s}: {row["g_drift_est_mugal"]:12.2f} µGal (sig = {row["sig_g_drift_est_mugal"]:5.2f} µGal)\n'
            if verbose:
                print(tmp_str)
            if self.write_log:
                self.log_str += tmp_str

        # ##### 3.) Determination of absolute gravity values #####
        sig_reading = 2  # [µGal], Assumption of the reading error, independent of the gravimeter type, etc.

        # Get datum stations:
        filter_tmp = self.stat_obs_df['is_datum'] == True
        stat_df_datum = self.stat_obs_df.loc[filter_tmp, :].copy(deep=True)
        stat_df_datum['sd_g_mugal'] = stat_df_datum['sd_g_mugal'].astype(float)  # Convert to float
        stat_df_datum['g_mugal'] = stat_df_datum['g_mugal'].astype(float)  # Convert to float
        number_of_datum_stations = len(stat_df_datum)

        # Differences between drift-corrected readings and datum g-values:
        stat_df_datum['g0_mugal'] = stat_df_datum['g_mugal'] - stat_df_datum['g_drift_est_mugal']
        stat_df_datum['sig_g0_mugal'] = np.sqrt(stat_df_datum['sig_g_drift_est_mugal'] ** 2
                                                + stat_df_datum['sd_g_mugal'] ** 2
                                                + sig_reading ** 2)

        # Calculation of the weighted mean of all g0:
        stat_df_datum['go_p'] = 1 / (stat_df_datum['sig_g0_mugal'] ** 2)
        g0_mean_mugal = sum(stat_df_datum['go_p'] * stat_df_datum['g0_mugal']) / sum(
            stat_df_datum['go_p'])  # Weighted mean of g0 values (weights: go_p)
        if number_of_datum_stations == 1:  # Only one OEGSN stations measured
            sig_g0_mean_mugal = stat_df_datum['sd_g_mugal']
        else:
            sig_g0_mean_mugal = np.sqrt(
                ((sum(stat_df_datum['g0_mugal'] ** 2) - ((sum(stat_df_datum['g0_mugal'])) ** 2 / number_of_datum_stations)) /
                 (number_of_datum_stations * (number_of_datum_stations - 1))) + (1 / (sum(stat_df_datum['go_p']))))

        # Calculation of the absolute g values at all observed stations:
        self.stat_obs_df['g_est_mugal'] = self.stat_obs_df['g_drift_est_mugal'] + g0_mean_mugal
        self.stat_obs_df['sd_g_est_mugal'] = np.sqrt(sig_g0_mean_mugal ** 2 + self.stat_obs_df['sig_g_drift_est_mugal'] ** 2)
        self.stat_obs_df['diff_g_est_mugal'] = self.stat_obs_df['g_est_mugal'] - self.stat_obs_df['g_mugal']
        self.stat_obs_df['diff_sd_g_est_mugal'] = self.stat_obs_df['sd_g_est_mugal'] - self.stat_obs_df['sd_g_mugal']
        # self.stat_obs_df['g_est_full_mugal'] = self.stat_obs_df['g_est_mugal'] + 9.8e8  # Get absolute gravity value at station [µGal]

        # Add g0 and sig_go to "self.stat_obs_df":
        self.stat_obs_df['g0_mugal'] = np.NAN
        self.stat_obs_df['sig_g0_mugal'] = np.NAN
        self.stat_obs_df.update(stat_df_datum[['g0_mugal', 'sig_g0_mugal']])

        # Write log:
        if verbose or self.write_log:
            tmp_str = f'\n'
            tmp_str += f'#### Absolute gravity determination ####\n'
            tmp_str += f'\n'
            tmp_str += f'Number of datum stations: {number_of_datum_stations}\n'
            tmp_str += self.stat_obs_df[['station_name', 'g_est_mugal', 'sd_g_est_mugal',
                                         'diff_g_est_mugal']].to_string(
                index=False,float_format=lambda x: '{:.1f}'.format(x))
            if verbose:
                print(tmp_str)
            if self.write_log:
                self.log_str += tmp_str

        # ##### 4.) Save data/infos for later use #####
        self.observed_stations = observed_stations
        self.drift_polynomial_degree = drift_pol_degree
        # self.number_of_estimates = number_of_parameters
        self.number_of_stations = number_of_stations
        # self.degree_of_freedom = dof

        # Drift dataframe:
        survey_name_list = []
        degree_list = []
        coefficient_list = []
        sd_coeff_list = []
        coeff_unit_list = []
        ref_epoch_t0_dt_list = []
        for degree, pol_coeff in pol_coef_dict.items():
            survey_name_list.append(None)  # One drift polynomial estimated for all surveys in campaign
            degree_list.append(degree)
            coefficient_list.append(pol_coeff)
            sd_coeff_list.append(None)  # No standard deviation available
            coeff_unit_list.append(f'µGal/h^{degree}')
            ref_epoch_t0_dt_list.append(ref_epoch_t0_dt)
        self.drift_pol_df = pd.DataFrame(list(zip(survey_name_list,
                                                  degree_list,
                                                  coefficient_list,
                                                  sd_coeff_list,
                                                  coeff_unit_list,
                                                  ref_epoch_t0_dt_list)),
                                         columns=self._DRIFT_POL_DF_COLUMNS)

        # Rename columns to be compatible with the table view model:
        all_setups_df.rename(columns={'epoch_dt': 'ref_epoch_dt'}, inplace=True)
        self.setup_obs_df = all_setups_df
        self.drift_ref_epoch_type = 'campaign'

        # Not used here => =None as initialized!
        # self.sig02_a_priori = sig0_mugal
        # self.scaling_factor_datum_observations = scaling_factor_datum_observations
        # self.confidence_level_chi_test = confidence_level_chi_test
        # self.confidence_level_tau_test = confidence_level_chi_test
        # self.s02_a_posteriori = s02_a_posteriori_mugal2





"""
Reading data from files and preparing the data for further processing is handled here.
This module is part of the gravtools package.
Author: Andreas Hellerschmied
"""

import sys

import pandas as pd
import numpy as np

# Imports from other gravtools modules:
from bev_legacy import const
from bev_legacy import settings
from bev_legacy import utils


def read_oesgn_table(filename, filepath=''):
    """
    Read ÖSGN Table and return pandas dataframe containing the data.
    :return:
    """
    widths = [
        10,  # Punktnummer im ÖSGN
        24,  # Anmerkung
        8,  # Geograph. Breite [deg] 34
        8,  # Geograph. Länge [deg] 42
        8,  # Höhe [m] 50
        7,  # g [µGal] 58
        3,  # mittlerer Fehler von g [?] 65
        4,  # Vertikalgradient der Schwere [µGal/m] 68
        6,  # Datum 73
        12,  # Punktidentität 79
    ]

    column_names = [
        'punktnummer',
        'anmerkungen',
        'breite_deg',
        'laenge_deg',
        'hoehe_m',
        'g_oesgn_mugal',
        'g_sig_oesgn_mugal',
        'vg_mugal',
        'date',
        'identitaet'
    ]

    df = pd.read_fwf(filepath + filename, widths=widths, header=None)
    df.columns = column_names

    return df


def read_obs_file(filename, filepath=''):
    """
    Einlesen der (konvertierten) Messdateien mit dem folgenden Format:
    Zeile 1: Für skalierte bzw. nicht skalierte Werte ('Y' bzw. 'N')
    Zeile 2: Gerätenummer (3 bzw. 5 für CG3 bzw. CG5)
    Zeile 3: Grad des geschätzten Polynoms (1, 2 oder 3)
    Zeile 4: Datum (YYYY MM DD)
    Zeile 5: Zeitzone (UTC, MEZ, OEZ)
    Zeile 6 bis n: Punktnummer(x-xxx-xx), Messzeit (hh.mm), g [mGal], Breite [°] (bb.b), Länge [°] (ll.l)
    Zeile n+1: 'end'

    return: dict with observation data
    """

    obs_data = dict()

    # Check, if the input file contains an additional columns with the tidal correction:
    flag_tide_corr = False
    if filename.split('_')[-1] == 'tideCorr':
        flag_tide_corr = True

    # Check, if the input file contains an additional columns with the tidal correction:
    flag_sd = False
    if filename.split('_')[-1] == 'sd':
        flag_sd = True

    # Read 5 header lines
    num_of_header_lines = 5
    with open(filepath + filename) as myfile:
        head = [next(myfile) for x in range(num_of_header_lines)]
    skalierung = head[0][:-1]
    instrument_id = head[1].split()[0]
    polynomial_degree = int(head[2][:-1])
    date_str = head[3][:-1]
    timezone_str = head[4][:-1]

    # Read obs data => pandas dataframe
    widths = [
        11,  # Punktnummer im ÖSGN
        5,  # Zeit
        9,  # g [mGal]
        7,  # dhb [cm]
        6,  # dhf [cm]
    ]
    column_names = [
        'punktnummer',
        'zeit',
        'g_mugal',
        'dhb_m',
        'dhf_m',
    ]

    if flag_tide_corr:
        widths.append(8)
        column_names.append('tide_mugal')

    if flag_sd:
        widths.append(6)
        column_names.append('g_sd_mugal')

    df = pd.read_fwf(filepath + filename, widths=widths, header=None, names=column_names, skiprows=5, skipfooter=1,
                     dtype={'zeit': object})
    if not flag_sd:
        df['g_sd_mugal'] = np.nan
    if not flag_tide_corr:
        df['tide_mugal'] = np.nan

    df['zeit'] = pd.to_datetime(date_str + ' ' + df['zeit'] + ' ' + timezone_str, format='%Y %m %d %H.%M %Z')
    df = df.set_index('zeit')  # set 'zeit' as index
    df['g_mugal'] = df['g_mugal'] * 1e3  # Conversion from mGal to muGal
    df['tide_mugal'] = df['tide_mugal'] * 1e3  # Conversion from mGal to muGal
    df['g_sd_mugal'] = df['g_sd_mugal'] * 1e3  # Conversion from mGal to muGal
    df['dhb_m'] = df['dhb_m'] * 1e-2  # Conversion from cm to m
    df['dhf_m'] = df['dhf_m'] * 1e-2  # Conversion from cm to m

    df['gravimeter_typ'] = settings.GRAVIMETER_ID_BEV[instrument_id]

    # Store results in dict:
    obs_data['skalierung'] = skalierung
    obs_data['instrument_id'] = instrument_id
    obs_data['polynomial_degree'] = polynomial_degree
    obs_data['date_str'] = date_str
    obs_data['timezone_str'] = timezone_str
    obs_data['obs_df'] = df
    obs_data['filename'] = filename

    return obs_data


def prep_obs_df(obs_df, df_oesgn):
    """
    Dataframe mit Beobachtungen aufbereiten und mit Informationen aus dm ÖSGN erweitern.
    :param obs_df: dataframe mit Messungen
    :param df_oesgn: dataframe mit ÖSGN Daten
    :return: dataframe mit Messungen (erweitert)
    """

    # Mit ÖSGN Tabelle mergen und die Messepoche als Index setzen:
    obs_df = obs_df.reset_index().merge(df_oesgn.drop(columns=['anmerkungen', 'identitaet', 'date', 'laenge_deg',
                                                               'hoehe_m', 'breite_deg']), on='punktnummer',
                                        how='left').set_index('zeit')  # Index = Zeit

    # Initialize columns that are to be filled later on:
    obs_df['g_red_mugal'] = np.nan  # Auf Festpunkt reduzierter Schwerewert [µGal]
    obs_df['corr_drift_mugal'] = np.nan  # Auf Drift-Korrektur für Lesung [µGal]
    obs_df['abw_mugal'] = np.nan  # Abweichung zwischen Drift korrigierter Lesung und Schätzwert g_est am Punkt

    # Zeitliche Referenz der Messungen:
    obs_df['unix_time'] = pd.to_numeric(obs_df.index)  # Unix Zeit (absolut) = ns since 1970
    obs_df['dt_h'] = (obs_df.index - obs_df.index.min()).seconds / 3600.0  # Zeitspannt zw. Messung i = 1 bis n und
    # erster Messung i=0

    # Falls VG nicht über die ÖSGN Tabelle gegeben, default Wert verwenden:
    obs_df.loc[obs_df.vg_mugal.isna(), 'vg_mugal'] = const.VG_DEFAULT

    return obs_df


def get_station_df(obs_df, df_oesgn):
    """
    Erzeugt einen dataframe mit allen Stationen um stationsspezifische Daten und Ergebnisse zu speichern.
    :param obs_df: dataframe mit Messungen
    :param df_oesgn: dataframe mit ÖSGN Daten
    :return: stat_df: dict with observation data
    """

    # get unique station list and number of observations per station:
    tmp_series = obs_df.punktnummer.value_counts()
    stat_df = tmp_series.to_frame(name='num_of_obs')
    stat_df = stat_df.reset_index().rename(columns={'index': 'punktnummer'})

    # Initialize columns that are to be filled later on:
    stat_df['is_drift_point'] = stat_df.num_of_obs > 1  # More than 1 obs => Drif point!
    stat_df['g_est_mugal'] = np.nan
    stat_df['sig_g_est_mugal'] = np.nan
    stat_df['dhb_m'] = np.nan  # Depends on the setup of each measurement => Only the difference (dhb-dhf) is const.!
    stat_df['dhf_m'] = np.nan  # Depends on the setup of each measurement => Only the difference (dhb-dhf) is const.!
    stat_df['g_abs_mugal'] = np.nan
    stat_df['sig_g_abs_mugal'] = np.nan
    stat_df['verb_mugal'] = np.nan
    stat_df['g_abs_full_mugal'] = np.nan  # absolute value in µGal incl. 9.8e8
    stat_df['g0_mugal'] = np.nan  # Difference between g_ÖSGN and estimated g_rel
    stat_df['sig_g0_mugal'] = np.nan  # Std.Dev. of difference between g_ÖSGN and estimated g_rel

    # Add ÖSGN data:
    stat_df = stat_df.merge(df_oesgn.drop(columns=['anmerkungen', 'identitaet', 'date']), on='punktnummer', how='left')
    stat_df['is_oesgn'] = ~stat_df.g_oesgn_mugal.isna()
    stat_df['date'] = obs_df.index[0].date()  # date for observation

    # Assign dhb and dhf:
    for stat in stat_df.punktnummer:
        stat_df.loc[stat_df['punktnummer'] == stat, 'dhb_m'] = \
            obs_df[obs_df['punktnummer'] == stat].dhb_m[-1]  # Assign dhb of the last obervation
        stat_df.loc[stat_df['punktnummer'] == stat, 'dhf_m'] = \
            obs_df[obs_df['punktnummer'] == stat].dhf_m[-1]  # Assign dhb of the last obervation

    return stat_df



def read_and_prep_data(path_oesgn_table, name_oesgn_table, path_obs_file, name_obs_file):
    # ##### Load data #####
    # ÖSGN Tabelle laden:
    if settings.VERBOSE:
        print('Datei einlesen: {}'.format(path_oesgn_table + name_oesgn_table))
    df_oesgn = read_oesgn_table(path_oesgn_table + name_oesgn_table)
    if settings.VERBOSE:
        print(' - {} OESGN Stationen geladen.'.format(df_oesgn.shape[0]))
        # print(df_oesgn.info())
    # Messdatei laden:
    if settings.VERBOSE:
        print('Datei einlesen: {}'.format(path_obs_file + name_obs_file))
    obs_dict = read_obs_file(name_obs_file, path_obs_file)
    # dataframe für Berechnung erstellen:
    obs_df = obs_dict['obs_df']
    del obs_dict['obs_df']
    obs_info_dict = obs_dict
    del obs_dict

    if settings.VERBOSE:
        print(' - {} Beobachtungen geladen.'.format(obs_df.shape[0]))
        # print(obs_dict['obs_df'].info())

    # ### Korrektur der Höhenoffsets vom Instrument zum Boden bzw. Festpunkt (dHF, dHB): ###
    obs_df = utils.corr_instrument_ref_heights(obs_df)

    # ### Dataframe mit Stationsliste erzeugen: ###
    stat_df = get_station_df(obs_df, df_oesgn)
    if settings.VERBOSE:
        print('Stations info:')
        print(' - Drift Stationen:   {}'.format(stat_df[stat_df['is_drift_point'] == True].shape[0]))
        print(' - ÖSGN Stationen:    {}'.format(stat_df[stat_df['is_oesgn'] == True].shape[0]))
        print(' - Stationen gesamt:  {}'.format(stat_df.shape[0]))
        print(' - Mittlere Breite:   {:6.3f}°'.format(stat_df.breite_deg.mean()))
        print(' - Mittlere Länge:    {:6.3f}°'.format(stat_df.laenge_deg.mean()))

    # ### Dataframe mit Messungen aufbereiten: ###
    obs_df = prep_obs_df(obs_df, df_oesgn)

    # ### Messwert mittels Vertikalgadienten auf die Höhe des Festpunktes reduzieren (mittls dHF) ###
    obs_df = utils.red_g_obs_vg(obs_df)
    if settings.VERBOSE:
        print('Gravimeter-Lesungen mittels VG auf Festpunkt-Niveau reduziert.')

    return obs_df, stat_df, df_oesgn, obs_info_dict


# Run as standalone program:
if __name__ == "__main__":

    if len(sys.argv) == 1:
        print('Eingangsparameter aus options.py bezogen (Default-Parameter).')
        path_oesgn_table = settings.PATH_OESGN_TABLE
        name_oesgn_table = settings.NAME_OESGN_TABLE
        path_obs_file = settings.PATH_OBS_FILE_BEV
        name_obs_file = settings.NAME_OBS_FILE_BEV

    elif len(sys.argv) == 2:  # Name of obervation file as input argument
        name_obs_file = sys.argv[1]
        # Default-parameters from options-file:
        path_oesgn_table = settings.PATH_OESGN_TABLE
        name_oesgn_table = settings.NAME_OESGN_TABLE
        path_obs_file = settings.PATH_OBS_FILE_BEV

    else:
        print('Error: Invalid number of input arguments!')
        exit()

    # Run init module:
    obs_df, stat_df, oesgn_df, obs_info_dict = read_and_prep_data(path_oesgn_table,
                                                                  name_oesgn_table,
                                                                  path_obs_file,
                                                                  name_obs_file)
    # Print dataframe info:
    print('\n--- Observation dataframe ---')
    obs_df.info()
    print('\n--- Stations dataframe ---')
    stat_df.info()
    print('\n--- ÖSGN dataframe ---')
    oesgn_df.info()

else:
    # not run as standalone program, but as module
    pass



"""Dialog for managing observation corrections.

Copyright (C) 2023  Andreas Hellerschmied <andreas.hellerschmied@bev.gv.at>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

from PyQt5.QtWidgets import QDialog, QMessageBox
from PyQt5 import QtCore

from gravtools.gui.dialog_corrections import Ui_Dialog_corrections
from gravtools import settings

class DialogCorrections(QDialog, Ui_Dialog_corrections):
    """Dialog to select and apply observation corrections."""

    def __init__(self, parent=None):
        super().__init__(parent)

        # Run the .setupUi() method to show the GUI
        self.setupUi(self)
        # connect signals and slots:

        # Other inits:

        # Set up combo box for interpolation method selection and select default method:
        for idx, (method_name, tooltip) in enumerate(settings.SCIPY_INTERP1_INTERPOLATION_METHODS.items()):
            self.comboBox_tides_interpolation_method.addItem(method_name)
            self.comboBox_tides_interpolation_method.setItemData(idx, tooltip, QtCore.Qt.ToolTipRole)
            if method_name == settings.SCIPY_INTERP1_INTERPOLATION_DEFAULT_METHOD:
                self.comboBox_tides_interpolation_method.setCurrentIndex(idx)

        self.doubleSpinBox_atm_pres_admittance.setValue(settings.ATM_PRES_CORRECTION_ADMITTANCE_DEFAULT)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'gravtools/gui/dialog_correction_time_series.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_DialogCorrectionTimeSeries(object):
    def setupUi(self, DialogCorrectionTimeSeries):
        DialogCorrectionTimeSeries.setObjectName("DialogCorrectionTimeSeries")
        DialogCorrectionTimeSeries.resize(1214, 600)
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(DialogCorrectionTimeSeries)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.splitter = QtWidgets.QSplitter(DialogCorrectionTimeSeries)
        self.splitter.setOrientation(QtCore.Qt.Horizontal)
        self.splitter.setHandleWidth(4)
        self.splitter.setChildrenCollapsible(False)
        self.splitter.setObjectName("splitter")
        self.layoutWidget = QtWidgets.QWidget(self.splitter)
        self.layoutWidget.setObjectName("layoutWidget")
        self.verticalLayout_left = QtWidgets.QVBoxLayout(self.layoutWidget)
        self.verticalLayout_left.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.verticalLayout_left.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_left.setObjectName("verticalLayout_left")
        self.treeWidget_selection = QtWidgets.QTreeWidget(self.layoutWidget)
        self.treeWidget_selection.setHeaderHidden(True)
        self.treeWidget_selection.setColumnCount(1)
        self.treeWidget_selection.setObjectName("treeWidget_selection")
        self.treeWidget_selection.headerItem().setText(0, "1")
        self.treeWidget_selection.header().setVisible(False)
        self.verticalLayout_left.addWidget(self.treeWidget_selection)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.pushButton_delete_selection = QtWidgets.QPushButton(self.layoutWidget)
        self.pushButton_delete_selection.setObjectName("pushButton_delete_selection")
        self.horizontalLayout_2.addWidget(self.pushButton_delete_selection)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.verticalLayout_left.addLayout(self.horizontalLayout_2)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.pushButton_collapse = QtWidgets.QPushButton(self.layoutWidget)
        self.pushButton_collapse.setObjectName("pushButton_collapse")
        self.horizontalLayout_3.addWidget(self.pushButton_collapse)
        self.pushButton_expand = QtWidgets.QPushButton(self.layoutWidget)
        self.pushButton_expand.setObjectName("pushButton_expand")
        self.horizontalLayout_3.addWidget(self.pushButton_expand)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem1)
        self.verticalLayout_left.addLayout(self.horizontalLayout_3)
        self.layoutWidget1 = QtWidgets.QWidget(self.splitter)
        self.layoutWidget1.setObjectName("layoutWidget1")
        self.verticalLayout_right = QtWidgets.QVBoxLayout(self.layoutWidget1)
        self.verticalLayout_right.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.verticalLayout_right.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_right.setObjectName("verticalLayout_right")
        self.stackedWidget_item_details = QtWidgets.QStackedWidget(self.layoutWidget1)
        self.stackedWidget_item_details.setObjectName("stackedWidget_item_details")
        self.page_empty = QtWidgets.QWidget()
        self.page_empty.setObjectName("page_empty")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.page_empty)
        self.verticalLayout.setObjectName("verticalLayout")
        self.label_6 = QtWidgets.QLabel(self.page_empty)
        self.label_6.setObjectName("label_6")
        self.verticalLayout.addWidget(self.label_6)
        spacerItem2 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem2)
        self.stackedWidget_item_details.addWidget(self.page_empty)
        self.page_survey = QtWidgets.QWidget()
        self.page_survey.setObjectName("page_survey")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.page_survey)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.formLayout = QtWidgets.QFormLayout()
        self.formLayout.setObjectName("formLayout")
        self.label = QtWidgets.QLabel(self.page_survey)
        self.label.setObjectName("label")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label)
        self.label_survey_name = QtWidgets.QLabel(self.page_survey)
        self.label_survey_name.setText("")
        self.label_survey_name.setObjectName("label_survey_name")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.label_survey_name)
        self.label_2 = QtWidgets.QLabel(self.page_survey)
        self.label_2.setObjectName("label_2")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.label_2)
        self.label_num_of_stations = QtWidgets.QLabel(self.page_survey)
        self.label_num_of_stations.setText("")
        self.label_num_of_stations.setObjectName("label_num_of_stations")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.label_num_of_stations)
        self.verticalLayout_5.addLayout(self.formLayout)
        self.stackedWidget_item_details.addWidget(self.page_survey)
        self.page_station = QtWidgets.QWidget()
        self.page_station.setObjectName("page_station")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.page_station)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.formLayout_2 = QtWidgets.QFormLayout()
        self.formLayout_2.setObjectName("formLayout_2")
        self.label_3 = QtWidgets.QLabel(self.page_station)
        self.label_3.setObjectName("label_3")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.label_3)
        self.label_station_data_source = QtWidgets.QLabel(self.page_station)
        self.label_station_data_source.setText("")
        self.label_station_data_source.setObjectName("label_station_data_source")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.label_station_data_source)
        self.label_4 = QtWidgets.QLabel(self.page_station)
        self.label_4.setObjectName("label_4")
        self.formLayout_2.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.label_4)
        self.label_station_description = QtWidgets.QLabel(self.page_station)
        self.label_station_description.setText("")
        self.label_station_description.setObjectName("label_station_description")
        self.formLayout_2.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.label_station_description)
        self.label_5 = QtWidgets.QLabel(self.page_station)
        self.label_5.setObjectName("label_5")
        self.formLayout_2.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.label_5)
        self.label_station_unit = QtWidgets.QLabel(self.page_station)
        self.label_station_unit.setText("")
        self.label_station_unit.setObjectName("label_station_unit")
        self.formLayout_2.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.label_station_unit)
        self.label_10 = QtWidgets.QLabel(self.page_station)
        self.label_10.setObjectName("label_10")
        self.formLayout_2.setWidget(5, QtWidgets.QFormLayout.LabelRole, self.label_10)
        self.label_station_model_type = QtWidgets.QLabel(self.page_station)
        self.label_station_model_type.setText("")
        self.label_station_model_type.setObjectName("label_station_model_type")
        self.formLayout_2.setWidget(5, QtWidgets.QFormLayout.FieldRole, self.label_station_model_type)
        self.label_7 = QtWidgets.QLabel(self.page_station)
        self.label_7.setObjectName("label_7")
        self.formLayout_2.setWidget(6, QtWidgets.QFormLayout.LabelRole, self.label_7)
        self.label_station_start = QtWidgets.QLabel(self.page_station)
        self.label_station_start.setText("")
        self.label_station_start.setObjectName("label_station_start")
        self.formLayout_2.setWidget(6, QtWidgets.QFormLayout.FieldRole, self.label_station_start)
        self.label_8 = QtWidgets.QLabel(self.page_station)
        self.label_8.setObjectName("label_8")
        self.formLayout_2.setWidget(7, QtWidgets.QFormLayout.LabelRole, self.label_8)
        self.label_station_end = QtWidgets.QLabel(self.page_station)
        self.label_station_end.setText("")
        self.label_station_end.setObjectName("label_station_end")
        self.formLayout_2.setWidget(7, QtWidgets.QFormLayout.FieldRole, self.label_station_end)
        self.label_9 = QtWidgets.QLabel(self.page_station)
        self.label_9.setObjectName("label_9")
        self.formLayout_2.setWidget(8, QtWidgets.QFormLayout.LabelRole, self.label_9)
        self.label_station_duration = QtWidgets.QLabel(self.page_station)
        self.label_station_duration.setText("")
        self.label_station_duration.setObjectName("label_station_duration")
        self.formLayout_2.setWidget(8, QtWidgets.QFormLayout.FieldRole, self.label_station_duration)
        self.label_12 = QtWidgets.QLabel(self.page_station)
        self.label_12.setObjectName("label_12")
        self.formLayout_2.setWidget(9, QtWidgets.QFormLayout.LabelRole, self.label_12)
        self.label_station_nuber_of_datapoints = QtWidgets.QLabel(self.page_station)
        self.label_station_nuber_of_datapoints.setText("")
        self.label_station_nuber_of_datapoints.setObjectName("label_station_nuber_of_datapoints")
        self.formLayout_2.setWidget(9, QtWidgets.QFormLayout.FieldRole, self.label_station_nuber_of_datapoints)
        self.label_11 = QtWidgets.QLabel(self.page_station)
        self.label_11.setObjectName("label_11")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label_11)
        self.label_station_station_name = QtWidgets.QLabel(self.page_station)
        self.label_station_station_name.setText("")
        self.label_station_station_name.setObjectName("label_station_station_name")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.label_station_station_name)
        self.label_13 = QtWidgets.QLabel(self.page_station)
        self.label_13.setObjectName("label_13")
        self.formLayout_2.setWidget(4, QtWidgets.QFormLayout.LabelRole, self.label_13)
        self.label_station_created = QtWidgets.QLabel(self.page_station)
        self.label_station_created.setText("")
        self.label_station_created.setObjectName("label_station_created")
        self.formLayout_2.setWidget(4, QtWidgets.QFormLayout.FieldRole, self.label_station_created)
        self.verticalLayout_2.addLayout(self.formLayout_2)
        self.graphicsLayoutWidget_time_series = GraphicsLayoutWidget(self.page_station)
        self.graphicsLayoutWidget_time_series.setObjectName("graphicsLayoutWidget_time_series")
        self.verticalLayout_2.addWidget(self.graphicsLayoutWidget_time_series)
        self.stackedWidget_item_details.addWidget(self.page_station)
        self.verticalLayout_right.addWidget(self.stackedWidget_item_details)
        self.frame_actions = QtWidgets.QFrame(self.layoutWidget1)
        self.frame_actions.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame_actions.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame_actions.setObjectName("frame_actions")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.frame_actions)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.pushButton_load_tsf_file = QtWidgets.QPushButton(self.frame_actions)
        self.pushButton_load_tsf_file.setObjectName("pushButton_load_tsf_file")
        self.horizontalLayout.addWidget(self.pushButton_load_tsf_file)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem3)
        self.verticalLayout_4.addLayout(self.horizontalLayout)
        self.verticalLayout_right.addWidget(self.frame_actions)
        self.verticalLayout_3.addWidget(self.splitter)

        self.retranslateUi(DialogCorrectionTimeSeries)
        self.stackedWidget_item_details.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(DialogCorrectionTimeSeries)

    def retranslateUi(self, DialogCorrectionTimeSeries):
        _translate = QtCore.QCoreApplication.translate
        DialogCorrectionTimeSeries.setWindowTitle(_translate("DialogCorrectionTimeSeries", "Time series data"))
        self.pushButton_delete_selection.setToolTip(_translate("DialogCorrectionTimeSeries", "Delete selected item in the tree view."))
        self.pushButton_delete_selection.setText(_translate("DialogCorrectionTimeSeries", "Delete selected item"))
        self.pushButton_collapse.setToolTip(_translate("DialogCorrectionTimeSeries", "Toggle collapse tree view."))
        self.pushButton_collapse.setText(_translate("DialogCorrectionTimeSeries", "Collapse all"))
        self.pushButton_expand.setText(_translate("DialogCorrectionTimeSeries", "Expand all"))
        self.label_6.setText(_translate("DialogCorrectionTimeSeries", "<html><head/><body><p>← Select item in the tree widget to show details!</p></body></html>"))
        self.label.setToolTip(_translate("DialogCorrectionTimeSeries", "Survey name."))
        self.label.setText(_translate("DialogCorrectionTimeSeries", "Survey name:"))
        self.label_survey_name.setToolTip(_translate("DialogCorrectionTimeSeries", "Survey name."))
        self.label_2.setToolTip(_translate("DialogCorrectionTimeSeries", "Number of stations in the survey."))
        self.label_2.setText(_translate("DialogCorrectionTimeSeries", "Number of stations:"))
        self.label_num_of_stations.setToolTip(_translate("DialogCorrectionTimeSeries", "Number of stations in the survey."))
        self.label_3.setToolTip(_translate("DialogCorrectionTimeSeries", "Data source, e.g. file name."))
        self.label_3.setText(_translate("DialogCorrectionTimeSeries", "Data source:"))
        self.label_station_data_source.setToolTip(_translate("DialogCorrectionTimeSeries", "Data source, e.g. file name."))
        self.label_4.setToolTip(_translate("DialogCorrectionTimeSeries", "Description of the time series data."))
        self.label_4.setText(_translate("DialogCorrectionTimeSeries", "Description:"))
        self.label_station_description.setToolTip(_translate("DialogCorrectionTimeSeries", "Description of the time series data."))
        self.label_5.setToolTip(_translate("DialogCorrectionTimeSeries", "Unit."))
        self.label_5.setText(_translate("DialogCorrectionTimeSeries", "Unit:"))
        self.label_station_unit.setToolTip(_translate("DialogCorrectionTimeSeries", "Unit."))
        self.label_10.setToolTip(_translate("DialogCorrectionTimeSeries", "<html><head/><body><p>Indicates wheter the time series data represent the <span style=\" font-style:italic;\">effect</span> on the observed gravity, or <span style=\" font-style:italic;\">corrections</span> for the observed gravity (opposite signs!). </p></body></html>"))
        self.label_10.setText(_translate("DialogCorrectionTimeSeries", "Model type:"))
        self.label_station_model_type.setToolTip(_translate("DialogCorrectionTimeSeries", "<html><head/><body><p>Indicates wheter the time series data represent the <span style=\" font-style:italic;\">effect</span> on the observed gravity, or <span style=\" font-style:italic;\">corrections</span> for the observed gravity (opposite signs!). </p></body></html>"))
        self.label_7.setToolTip(_translate("DialogCorrectionTimeSeries", "Start date and time."))
        self.label_7.setText(_translate("DialogCorrectionTimeSeries", "Start:"))
        self.label_station_start.setToolTip(_translate("DialogCorrectionTimeSeries", "Start date and time."))
        self.label_8.setToolTip(_translate("DialogCorrectionTimeSeries", "<html><head/><body><p>End date and time.</p></body></html>"))
        self.label_8.setText(_translate("DialogCorrectionTimeSeries", "End:"))
        self.label_station_end.setToolTip(_translate("DialogCorrectionTimeSeries", "End date and time."))
        self.label_9.setToolTip(_translate("DialogCorrectionTimeSeries", "Duration"))
        self.label_9.setText(_translate("DialogCorrectionTimeSeries", "Duration:"))
        self.label_station_duration.setToolTip(_translate("DialogCorrectionTimeSeries", "Duration"))
        self.label_12.setToolTip(_translate("DialogCorrectionTimeSeries", "Total number of data points."))
        self.label_12.setText(_translate("DialogCorrectionTimeSeries", "No. datapoints:"))
        self.label_station_nuber_of_datapoints.setToolTip(_translate("DialogCorrectionTimeSeries", "Total number of data points."))
        self.label_11.setToolTip(_translate("DialogCorrectionTimeSeries", "Station name."))
        self.label_11.setText(_translate("DialogCorrectionTimeSeries", "Station name:"))
        self.label_station_station_name.setToolTip(_translate("DialogCorrectionTimeSeries", "Station name."))
        self.label_13.setToolTip(_translate("DialogCorrectionTimeSeries", "Date and time (UTC) when the time series data was loaded to GravTools."))
        self.label_13.setText(_translate("DialogCorrectionTimeSeries", "Created (UTC):"))
        self.label_station_created.setToolTip(_translate("DialogCorrectionTimeSeries", "Date and time (UTC) when the time series data was loaded to GravTools."))
        self.graphicsLayoutWidget_time_series.setToolTip(_translate("DialogCorrectionTimeSeries", "Plot of the time series."))
        self.pushButton_load_tsf_file.setToolTip(_translate("DialogCorrectionTimeSeries", "Load TSF file (e.g. createb by TSoft)"))
        self.pushButton_load_tsf_file.setText(_translate("DialogCorrectionTimeSeries", "Load TSF file"))
from pyqtgraph import GraphicsLayoutWidget
